import 'dart:convert';

class Address {
  /// The city name for the address, for example, "Grosswallstadt".
  String? city;

  /// An ISO-3166-1 (3-letter) country code for the address, for example, "DE".
  String? countryCode;

  /// The country name for the address, for example, "Deutschland".
  String? country;

  /// The district name for the address.
  /// It is a division of city, typically an administrative unit within a larger city or
  /// a customary name of a city's neighborhood, for example, "Bedford-Stuyvesant".
  String? district;

  /// The subdistrict name for the address.
  /// It is a subdivision of a district.
  String? subdistrict;

  /// The house name or number for the address, for example, "7".
  String? houseNumberOrName;

  /// The postal code for the address.
  /// It is an alphanumeric string included in a postal address to facilitate mail sorting, known locally
  /// in various countries throughout the world as a postcode, post code, PIN or ZIP Code, for example, "63868".
  String? postalCode;

  /// The state name for the address.
  /// It is code/abbreviation for the state division of a country, for example, "Bayern".
  String? state;

  /// The county name for the address.
  /// It is a division of a state, typically a secondary-level administrative division of a country or equivalent,
  /// for example, "Kings".
  String? county;

  /// The street name for the address, for example, "Industriering".
  String? street;

  Address({
    this.city,
    this.countryCode,
    this.country,
    this.district,
    this.subdistrict,
    this.houseNumberOrName,
    this.postalCode,
    this.state,
    this.county,
    this.street,
  });

  Address copyWith({
    String? city,
    String? countryCode,
    String? country,
    String? district,
    String? subdistrict,
    String? houseNumberOrName,
    String? postalCode,
    String? state,
    String? county,
    String? street,
  }) {
    return Address(
      city: city ?? this.city,
      countryCode: countryCode ?? this.countryCode,
      country: country ?? this.country,
      district: district ?? this.district,
      subdistrict: subdistrict ?? this.subdistrict,
      houseNumberOrName: houseNumberOrName ?? this.houseNumberOrName,
      postalCode: postalCode ?? this.postalCode,
      state: state ?? this.state,
      county: county ?? this.county,
      street: street ?? this.street,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'city': city,
      'countryCode': countryCode,
      'country': country,
      'district': district,
      'subdistrict': subdistrict,
      'houseNumberOrName': houseNumberOrName,
      'postalCode': postalCode,
      'state': state,
      'county': county,
      'street': street,
    };
  }

  factory Address.fromMap(Map<String, dynamic> map) {
    return Address(
      city: map['city'],
      countryCode: map['countryCode'],
      country: map['country'],
      district: map['district'],
      subdistrict: map['subdistrict'],
      houseNumberOrName: map['houseNumberOrName'],
      postalCode: map['postalCode'],
      state: map['state'],
      county: map['county'],
      street: map['street'],
    );
  }

  String toJson() => json.encode(toMap());

  factory Address.fromJson(String source) => Address.fromMap(json.decode(source));

  @override
  String toString() {
    return 'Address(city: $city, countryCode: $countryCode, country: $country, district: $district, subdistrict: $subdistrict, houseNumberOrName: $houseNumberOrName, postalCode: $postalCode, state: $state, county: $county, street: $street)';
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is Address &&
        other.city == city &&
        other.countryCode == countryCode &&
        other.country == country &&
        other.district == district &&
        other.subdistrict == subdistrict &&
        other.houseNumberOrName == houseNumberOrName &&
        other.postalCode == postalCode &&
        other.state == state &&
        other.county == county &&
        other.street == street;
  }

  @override
  int get hashCode {
    return city.hashCode ^
        countryCode.hashCode ^
        country.hashCode ^
        district.hashCode ^
        subdistrict.hashCode ^
        houseNumberOrName.hashCode ^
        postalCode.hashCode ^
        state.hashCode ^
        county.hashCode ^
        street.hashCode;
  }

  String getStreetAndNumber() {
    return _joinWithSeparator(street, houseNumberOrName);
  }

  String getZipAndCity() {
    return _joinWithSeparator(postalCode, city);
  }

  String _joinWithSeparator(String? first, String? second, [String separator = ' ']) {
    first ??= '';
    second = second == null ? '' : '$separator$second';
    return '$first$second';
  }

  bool get hasAddress {
    return city != null ||
        countryCode != null ||
        country != null ||
        district != null ||
        subdistrict != null ||
        houseNumberOrName != null ||
        postalCode != null ||
        state != null ||
        county != null ||
        street != null;
  }

  String asSingleLineString() {
    return _joinWithSeparator(getStreetAndNumber(), getZipAndCity(), ',');
  }
}
